####
# This is trying to test all that cmor_checker catches all the possible failures
###
from __future__ import print_function
import sys
import cmor
import shutil
import os
import cdms2
cdms2.setNetcdfShuffleFlag(0)
cdms2.setNetcdfDeflateFlag(0)
cdms2.setNetcdfDeflateLevelFlag(0)

fo = sys.stdout

if fo != sys.stdout:
    fo = open(fo, "w")

# First of all run the Test script that generates the "good" file
# execfile("Test/test_python_joerg_3.py")

file = 'Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc'


cmor.checkCMOR(fo, file, "Tables/CMIP5_6hrLev")


# Ok at this point we are going to test failures to make sure it actually checks for each things
# Ok testing the DRS

# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/huhs/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmfos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/16hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/noVolfdc2000/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10b/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIP5/output/INSTITUTEd_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIP5/outputt/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="Test/CMIpP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)


# try:
# F="Test/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# The following should pass, DRS test turned off
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# try:
# os.makedirs(os.path.split(F)[0])
# except:
# pass
# shutil.copy(file,F)
# print 'No Testing DRS'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
# os.remove(F)

# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_20f10010100-2010010218.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_20100g10100-2010010218.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010g100-2010010218.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_201001010g0-2010010218.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2f01001041 8.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010h010218.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010h418.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# F="Test/CMIP5/output/INSTITUTE_ID/pcmdi-10a/DcppC2/6hr/atmos/hus/r1i1p1/hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-201001041g8.nc"
# shutil.copy(file,F)
# print 'Testing DRS wrong date'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev")
##     raise "DRS Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
# f=cdms2.open(F,"a")
# h=f("hus")
# f.write(h,id='hur')
# f.close()
# print 'Testing many vars'
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise "Too man vars Check failed!"
# except cmor.check_CMOR_compliant.CMORError,err:
# print 'Failed with CMOR error as expected:',err
# pass
# except Exception,err:
##     raise Exception,err
# os.remove(F)

## gbl = ['branch_time','contact','Conventions','creation_date','experiment','experiment_id','forcing','frequency','initialization_method','institute_id','institution','model_id','modeling_realm','parent_experiment_id','physics_version','product','project_id','realization','source','table_id','tracking_id']
# for gatt in gbl:
# try:
# print 'Testing no %s global att' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##         os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##                 fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# f.close()
# fi.close()
# os.remove(F)
##         os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##         raise Exception,"No global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("File must have global attribute: %s" % (gatt))>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##             raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##         raise Exception,err
# os.remove(F)

# test validity of Creation Time
# try:
##     gatt = "creation_date"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-3]+'blabla" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
# print 'Did we get here?'
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("Creation Date must be in format: %Y-%m-%dT%H:%M:%SZ yours is")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "branch_time"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-4]+'"blabla" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("branch_time must be convertible to float, you have")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "branch_time"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-4]+'1. ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("if global attribute parent_experiment_id is N/A then branch_time must be 0., you have")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "experiment_id"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-3]+'x" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("experiment_id file attribute must be one of")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "experiment"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-3]+'x" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("experiment file attribute must be one of")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "forcing"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-3]+'x" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("file attribute forcing must be a comma separated list with values in")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "frequency"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-6]+'moyn" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("frequency must be one")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "frequency"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-6]+'mon" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("your file name indicates a frequency of ")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "parent_experiment_id"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-3]+'bla" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("parent_experiment_id file attribute must be one of")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "parent_experiment_id"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-6]+'DcppC2" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("parent_experiment_id and experiment_id cannot be the same")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     gatt = "project_id"
# print 'Testing Validity of %s' % (gatt)
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# fglb=False
# for l in fi.xreadlines():
# if l.find("// global attributes")>-1:
##             fglb = True
# if fglb is False:
# f.write(l)
# elif l.find(":%s" % gatt)==-1:
# f.write(l)
# else:
##             f.write(l.strip()[:-3]+'0" ;\n')
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong global attribute %s test failed" % (gatt)
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("project_id must be ")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of global attribute %s failed for the wrong reason! %s" % (gatt,err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# try:
# print 'Testing Validity of variable type'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for l in fi.xreadlines():
# if l.find("float hus(time")>-1:
# f.write(l.replace("float","double"))
##             fglb = True
# else:
# f.write(l)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong variable type test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("variable typecode must be ")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing Validity of coordinate type'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for l in fi.xreadlines():
# if l.find("double lon(")>-1:
# f.write(l.replace("double","float"))
##             fglb = True
# else:
# f.write(l)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong coord type test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("required typecode")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing Validity of coordinate type'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for l in fi.xreadlines():
# if l.find("double ap(")>-1:
# f.write(l.replace("double","float"))
##             fglb = True
# else:
# f.write(l)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong coord type test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("variable typecode")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing Validity of coordinate name'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for l in fi.xreadlines():
# if l.find("lat")>-1:
# if l.find('"latitude"')>-1:
# f.write(l.replace("lat","latitude",1))
# else:
# f.write(l.replace("lat","latitude"))
# else:
# f.write(l)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong coord name test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("Axis name latitude is not valid")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing Validity of coordinate units'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for l in fi.xreadlines():
# if l.find("lev:units")>-1:
# f.write(l.replace("1","%"))
# else:
# f.write(l)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong coord units test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find("are not the required units")>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
##     import cdms2
# print 'Testing Validity of time units'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
# f=cdms2.open(F,"r+")
# t=f['hus'].getTime()
##     t.toRelativeTime("seconds since 2010")
# f.close()
# f=open("Tables/CMIP5_6hrLev")
# fot=open("tmp_table.txt","w")
# for ln in f.xreadlines():
# if ln.find("approx_in")>-1:
# fot.write(ln.replace("0.25","21600."))
# else:
# fot.write(ln)
# fot.close()
# cmor.checkCMOR(fo,F,"tmp_table.txt",dodrs=False)
# os.remove("tmp_table.txt")
##     raise Exception,"wrong time units test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('Time units must be in "days since')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)


# data = """time_bnds =
##   -0.125, 0.125,
##   0.125, 0.375,
##   0.375, 0.625,
##   0.625, 0.875,
##   0.875, 1.125,
##   1.125, 1.375,
##   1.375, 1.625,
# 1.625, 1.875 ;"""
# data2 = """time_bnds =
##   -0.1251, 0.1251,
##   0.1251, 0.3751,
##   0.3751, 0.6251,
##   0.6251, 0.8751,
##   0.8751, 1.1251,
##   1.1251, 1.3751,
##   1.3751, 1.6251,
# 1.6251, 1.8751 ;"""
# try:
##     import cdms2
# print 'Testing time is mean of bounds'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# f.write(fi.read().replace(data,data2))
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"time is avg of bounds test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('Time units must be in "days since')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# data = """ time_bnds =
##   -0.125, 0.125,
##   0.125, 0.375,
##   0.375, 0.625,
##   0.625, 0.875,
##   0.875, 1.125,
##   1.125, 1.375,
##   1.375, 1.625,
# 1.625, 1.875 ;"""
# try:
##     import cdms2
# print 'Testing lack of bounds'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# s=fi.read()
# fi.close()
# fi=open("crap.txt",'w')
# fi.write(s.replace(data,""))
# fi.close()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("time:bounds")>-1:
# continue
# if ln.find("double time_bnds")>-1:
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"lack of bounds test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('attribute bounds is required for axis')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# data = """ time_bnds =
##   -0.125, 0.125,
##   0.125, 0.375,
##   0.375, 0.625,
##   0.625, 0.875,
##   0.875, 1.125,
##   1.125, 1.375,
##   1.375, 1.625,
# 1.625, 1.875 ;"""
# try:
##     import cdms2
# print 'Testing lack of bounds'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# s=fi.read()
# fi.close()
# fi=open("crap.txt",'w')
# fi.write(s.replace(data,""))
# fi.close()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("time:bounds")>-1:
# continue
# if ln.find("double time_bnds")>-1:
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"lack of bounds test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('attribute bounds is required for axis')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
## data="""double ap(lev) ;"""
# n=567
# data=data[:15]
# data2=""" ap = 0.1, 0.2, 0.3, 0.22, 0.1 ;

##  b = 0, 0.1, 0.2, 0.5, 0.8 ;

# ps ="""

# data3=""" ap_bnds =
##   0, 0.15,
##   0.15, 0.25,
##   0.25, 0.25,
##   0.25, 0.16,
##   0.16, 0 ;

# b_bnds =
##   0, 0.05,
##   0.05, 0.15,
##   0.15, 0.35,
##   0.35, 0.65,
##   0.65, 1 ;
# """
# try:
##     import cdms2
# print 'Testing lack of formulaterms'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# s=fi.read()
# fi.close()
# print 'orig:',len(s)
# i=s.find(data)
# print 'i,n:',i,n
# s=s[:i]+s[i+n:]
# s=s.replace(data2,"")
# s=s.replace(data3,"")
# print 'end:',len(s)
# fi=open("crap.txt",'w')
# fi.write(s)
# fi.close()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("formula")>-1:
# continue
# if ln.find("1013")>-1:
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"lack of formula_terms test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('attribute formula_terms is required for axis')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong names on zfactors axis for hybrid'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("ap")>-1:
# tmp=ln.replace("ap","ap2")
##             if tmp.find("formula_term")>-1: tmp=tmp.replace("ap2:","ap:")
# tmp=tmp.replace("ap2_bnds","ap_bnds")
# f.write(tmp)
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"lack of formula_terms test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('formula should be')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong names on stabdard_name axis for hybrid'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# ln=ln.replace("atmosphere_hybrid_sigma_pressure_coordinate","atmosphere_ln_pressure_coordinate")
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"lack of formula_terms test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('axis attribute formula should be')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# for anm in ['lon','lat','lev','time']:
# try:
# print 'Testing axis attribute'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("%s:axis"%anm)>-1:
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"no axis attribute test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('axis must have associated axis attribute')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing calendar attribute'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("time:calendar")>-1:
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"no calendar attribute test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('calendar attribute must be defined')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong calendar attribute'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("time:calendar")>-1:
##             f.write('time:calendar = "mayan" ;')
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"no calendar attribute test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('calendar must be one of')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong variable datatype'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("float hus")>-1:
##             f.write(ln.replace("float hus","double hus"))
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var typecode test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('variable typecode must be')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong variable name'
# F="HUS_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus")>-1:
# f.write(ln.replace("hus","HUS"))
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var name test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('does not start with standard CMIP5 variable name ')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong variable name'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus")>-1:
# f.write(ln.replace("hus","tas"))
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var name test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('Your file name says it contains variable')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong variable units'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus:units")>-1:
##             f.write('                hus:units = "%" ;\n')
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var name test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('do not match IPCC units')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong dim ordering'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("float hus(time, lev, lat, lon)")>-1:
##             f.write(ln.replace("lat, lon","lon, lat"))
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var ordering test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('in ordering for dimension')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)

# data = """ lon = 2, 6, 10, 14, 18, 22, 26, 30, 34, 38, 42, 46, 50, 54, 58, 62, 66, 70,
##     74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 134,
##     138, 142, 146, 150, 154, 158, 162, 166, 170, 174, 178, 182, 186, 190,
##     194, 198, 202, 206, 210, 214, 218, 222, 226, 230, 234, 238, 242, 246,
##     250, 254, 258, 262, 266, 270, 274, 278, 282, 286, 290, 294, 298, 302,
# 306, 310, 314, 318, 322, 326, 330, 334, 338, 342, 346, 350, 354, 358 ;"""
## datanew = """ lon = """+"""358, 354, 350, 346, 342, 338, 334, 330, 326, 322, 318, 314, 310, 306, 302, 298, 294, 290, 286, 282, 278, 274, 270, 266, 262, 258, 254, 250, 246, 242, 238, 234, 230, 226, 222, 218, 214, 210, 206, 202, 198, 194, 190, 186, 182, 178, 174, 170, 166, 162, 158, 154, 150, 146, 142, 138, 134, 130, 126, 122, 118, 114, 110, 106, 102, 98, 94, 90, 86, 82, 78, 74, 70, 66, 62, 58, 54, 50, 46, 42, 38, 34, 30, 26, 22, 18, 14, 10, 6, 2 ;"""
# try:
# print 'Testing wrong dim ordering'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# s=fi.read()
# fi.close()
# fi=open("crap.txt","w")
# fi.write(s.replace(data,datanew))
# fi.close()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var ordering test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('axis values for lon must be stored:increasingly')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong dim ordering'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("lon = 2, 6, 10"):
##             f.write(ln.replace("lon = 2, 6, 10","lon = -2, 6, 10"))
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var ordering test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('first longitude must be >= 0 degrees_east')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# data = """ lat = -88, -84, -80, -76, -72, -68, -64, -60, -56, -52, -48, -44, -40, -36,
##     -32, -28, -24, -20, -16, -12, -8, -4, 0, 4, 8, 12, 16, 20, 24, 28, 32,
# 36, 40, 44, 48, 52, 56, 60, 64, 68, 72, 76, 80, 84, 88 ;"""
# datanew = """ lat = """+"""88, 84, 80, 76, 72, 68, 64, 60, 56, 52, 48, 44, 40, 36,
##     32, 28, 24, 20, 16, 12, 8, 4, 0, -4, -8, -12, -16, -20, -24, -28, -32,
# -36, -40, -44, -48, -52, -56, -60, -64, -68, -72, -76, -80, -84, -88 ;"""
# try:
# print 'Testing wrong dim ordering'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# s=fi.read()
# fi.close()
# fi=open("crap.txt","w")
# fi.write(s.replace(data,datanew))
# fi.close()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var ordering test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('axis values for lat must be stored:increasingly')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
## data = """ time = 0, 0.25, 0.5, 0.75, 1, 1.25, 1.5, 1.75 ;"""
## datanew = """ time = 0, -0.25, -0.5, -0.75, -1, -1.25, -1.5, -1.75 ;"""

# try:
# print 'Testing wrong dim ordering'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2009123018.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# s=fi.read()
# fi.close()
# fi=open("crap.txt","w")
# fi.write(s.replace(data,datanew))
# fi.close()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var ordering test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('axis values for time must be stored:increasingly')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing fill and miss values different ordering'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus:mis")>-1:
##             f.write("                hus:missing_value = 1.e+30f ;\n")
# continue
# elif ln.find("hus:_F")>-1:
##             f.write("                hus:_FillValue = 1.e+20f ;\n")
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var ordering test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('missing_value and _FillValue attributes are different')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing fill and miss values different ordering'
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus:mis")>-1:
##             f.write("                hus:missing_value = 1.e+30f ;\n")
# continue
# elif ln.find("hus:_F")>-1:
##             f.write("                hus:_FillValue = 1.e+30f ;\n")
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"wrong var ordering test failed"
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('missing_value and _FillValue must be set to 1.e20f')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# for att in ['associated_files','cell_measures','long_name','standard_name','units']:
# try:
# print 'Testing required var att %s' % att
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##         os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus:%s"%att)>-1:
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##         os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##         raise Exception,"missing var att %s test failed" % att
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('Attribute %s is required but not set for var hus' % att)>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##             raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##         raise Exception,err
# os.remove(F)
## att = 'associated_files'
# try:
# print 'Testing wrong var att %s' % att
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus:%s"%att)>-1:
##             f.write('   hus:%s = "blabla" ; \n' % att)
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"missing var att %s test failed" % att
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('associated_files attributes must contain a baseURL')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
# try:
# print 'Testing wrong var att %s' % att
# F="hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
# shutil.copy(file,F)
##     os.popen("ncdump %s > crap.txt"%F).readlines()
# fi=open("crap.txt")
# f=open("crapo.txt","w")
# for ln in fi.xreadlines():
# if ln.find("hus:%s"%att)>-1:
##             f.write(' hus:associated_files = "baseURL: http://cmip-pcmdi.llnl.gov/CMIP5/dataLocation gridspecFile: gridspec_fx_pcmdi-10a_DcppC2_r0i0p0.nc cellAreaFile: areacellla_fx_pcmdi-10a_DcppC2_r0i0p0.nc" ;')
# continue
# f.write(ln)
# f.close()
# fi.close()
# os.remove(F)
##     os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
# os.remove("crap.txt")
# os.remove("crapo.txt")
# cmor.checkCMOR(fo,F,"Tables/CMIP5_6hrLev",dodrs=False)
##     raise Exception,"missing var att %s test failed" % att
# except cmor.check_CMOR_compliant.CMORError,err:
# if str(err).find('associated files should point to file containing: areacella')>-1:
# print 'Failed with CMOR error as expected:',err
# pass
# else:
##         raise Exception,"Checker of variable type failed for the wrong reason! %s" % (err)
# except Exception,err:
##     raise Exception,err
# os.remove(F)
att = "cell_measures"
try:
    print('Testing wrong var att %s' % att)
    F = "hus_6hrLev_pcmdi-10a_DcppC2_r1i1p1_2010010100-2010010218.nc"
    shutil.copy(file, F)
    os.popen("ncdump %s > crap.txt" % F).readlines()
    fi = open("crap.txt")
    f = open("crapo.txt", "w")
    for ln in fi:
        if ln.find("hus:%s" % att) > -1:
            f.write(' hus:%s = "area: cl" ;' % att)
            continue
        f.write(ln)
    f.close()
    fi.close()
    os.remove(F)
    os.popen("ncgen -b -o %s crapo.txt" % (F)).readlines()
    os.remove("crap.txt")
    os.remove("crapo.txt")
    cmor.checkCMOR(fo, F, "Tables/CMIP5_6hrLev", dodrs=False)
    raise Exception("missing var att %s test failed" % att)
except cmor.check_CMOR_compliant.CMORError as err:
    if str(err).find('variable attribute cell_measures should be') > -1:
        print('Failed with CMOR error as expected:', err)
        pass
    else:
        raise Exception(
            "Checker of variable type failed for the wrong reason! %s" %
            (err))
except Exception as err:
    raise Exception(err)
os.remove(F)



#!/usr/bin/env python2.7
# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/'
#      directories.
#
#   2. Download CMIP6_6hrLev.json, CMIP6_CV.json,
#      CMIP6_coordinate.json and CMIP6_formula_terms.json from
#      https://github.com/PCMDI/cmip6-cmor-tables/tree/master/Tables to
#      the 'Tables/' directory.
#
#   3. Download CMOR_input_example.json from
#      https://github.com/PCMDI/cmor/blob/master/Test/ to the 'Test/'
#      directory.
import numpy
import unittest
import cmor
import base_CMIP6_CV


class TestCase(base_CMIP6_CV.BaseCVsTest):
    def testCMIP6(self):
        try:
            inpath = 'Tables'  # 01.00.27b1
            cmor.setup(inpath=inpath, netcdf_file_action=cmor.CMOR_REPLACE,
                    logfile=self.tmpfile)
            error_flag = cmor.dataset_json('Test/CMOR_input_example.json')
            table_id = cmor.load_table('CMIP6_6hrLev_bad_specs.json')
            time = cmor.axis(table_entry='time1', units='days since 2000-01-01',
                            coord_vals=numpy.array(range(1)),
                            cell_bounds=numpy.array(range(2)))
            latitude = cmor.axis(table_entry='latitude', units='degrees_north',
                                coord_vals=numpy.array(range(5)),
                                cell_bounds=numpy.array(range(6)))
            longitude = cmor.axis(table_entry='longitude', units='degrees_east',
                                coord_vals=numpy.array(range(5)),
                                cell_bounds=numpy.array(range(6)))
            plev3 = cmor.axis(table_entry='plev3', units='Pa',
                            coord_vals=numpy.array([85000., 50000., 25000.]))
            axis_ids = [longitude, latitude, plev3, time]
            ua_var_id = cmor.variable(table_entry='ua', axis_ids=axis_ids,
                                    units='m s-1')
            ta_var_id = cmor.variable(table_entry='ta', axis_ids=axis_ids,
                                    units='K')
            data = numpy.random.random(75)
            reshaped_data = data.reshape((5, 5, 3, 1))
            
            # This doesn't:
            cmor.write(ta_var_id, reshaped_data)
            cmor.write(ua_var_id, reshaped_data)

            #cmor.close()
        except BaseException:
            pass

        self.assertCV("data_specs_version")

if __name__ == '__main__':
    unittest.main()


from __future__ import print_function
import common
import cmor
import os


pth = os.getcwd()
common.init_cmor(pth, "CMOR_input_example.json")
cmor.load_table(os.path.join(pth, 'Tables', 'CMIP6_6hrLev.json'))
itim, ilat, ilon = common.read_cmor_time1_lat_lon()

ilambda = cmor.axis(
    table_entry='lambda550nm',
    units='m',
    length=1,
    coord_vals=[550.,],
    cell_bounds=[500.,600.])
# Creates two singleton dims?

varid = cmor.variable(table_entry=common.specs["BS"]["entry"],
                      units=common.specs["BS"]["units"],
                      axis_ids=[itim, ilat, ilon, ilambda],
                      missing_value=1.e28,
                      positive=common.specs["BS"]["positive"],
                      original_name="BS")
for index in range(2):
    tim_array, bnds_tim = common.read_time(index)
    data = common.read_2d_input_files(index, "BS", (common.lat, common.lon))
    print(data.shape, data)
    print(tim_array, bnds_tim)
    cmor.write(var_id=varid, data=data, ntimes_passed=1,
               time_vals=tim_array, time_bnds=bnds_tim)
    print("Passed write")
cmor.close()



import cmor
import numpy

error_flag = cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)

error_flag = cmor.dataset_json("Test/CMOR_input_example.json")


cmor.load_table("CMIP6_Omon.json")
itime = cmor.axis(table_entry="time", units='months since 2010', coord_vals=numpy.array(
    [0, 1, 2, 3, 4.]), cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
# creates 1 degree grid
nlat = 18
nlon = 36
alats = numpy.arange(180) - 89.5
bnds_lat = numpy.arange(181) - 90
alons = numpy.arange(360) + .5
bnds_lon = numpy.arange(361)
ilat = cmor.axis(
    table_entry='latitude',
    units='degrees_north',
    length=nlat,
    coord_vals=alats,
    cell_bounds=bnds_lat)

ilon = cmor.axis(
    table_entry='longitude',
    length=nlon,
    units='degrees_east',
    coord_vals=alons,
    cell_bounds=bnds_lon)
ivar = cmor.variable(
    table_entry="eparag100",
    axis_ids=[
        itime,
        ilat,
        ilon],
    units='mol m-2 s-1',
    positive="up")

data = numpy.random.random((5, nlat, nlon))
for i in range(0, 5):
    # ,time_vals=numpy.array([i,]),time_bnds=numpy.array([i,i+1]))
    cmor.write(ivar, data[i:i])
error_flag = cmor.close()



# This is a dummy version of the ACCESS Post Processor.
# Peter Uhe 24 July 2014
# Martin Dix 21 Nov 2014
#
from __future__ import print_function
import numpy as np
import datetime
import cmor


def save(opts, threeD=True):

    cmor.setup(inpath=opts['table_path'],
               netcdf_file_action=cmor.CMOR_REPLACE_3,
               set_verbosity=cmor.CMOR_NORMAL,
               exit_control=cmor.CMOR_NORMAL,
               logfile=None, create_subdirectories=1)

    cmor.dataset_json("Test/CMOR_input_example.json")

    # Load the CMIP tables into memory.
    tables = []
    tables.append(cmor.load_table('CMIP6_grids.json'))
    tables.append(cmor.load_table(opts['cmip_table']))

    # Create the dimension axes

    # Monthly time axis
    min_tvals = []
    max_tvals = []
    cmor_tName = 'time'
    tvals = []
    axis_ids = []
    for year in range(1850, 1851):
        for mon in range(1, 13):
            tvals.append(datetime.date(year, mon, 15).toordinal() - 1)
    # set up time values and bounds
    for i, ordinaldate in enumerate(tvals):
        model_date = datetime.date.fromordinal(int(ordinaldate) + 1)
        # min bound is first day of month
        model_date = model_date.replace(day=1)
        min_tvals.append(model_date.toordinal() - 1)
        # max_bound is first day of next month
        tyr = model_date.year + int(model_date.month / 12)
        tmon = model_date.month % 12 + 1
        model_date = model_date.replace(year=tyr, month=tmon)
        max_tvals.append(model_date.toordinal() - 1)
        # correct date to middle of month
        mid = (max_tvals[i] - min_tvals[i]) / 2.
        tvals[i] = min_tvals[i] + mid
    tval_bounds = np.column_stack((min_tvals, max_tvals))
    cmor.set_table(tables[1])
    time_axis_id = cmor.axis(table_entry=cmor_tName,
                             units='days since 0001-01-01', length=len(tvals),
                             coord_vals=tvals[:], cell_bounds=tval_bounds[:],
                             interval=None)
    axis_ids.append(time_axis_id)

    if not threeD:
        # Pressure
        plev = np.array([100000, 92500, 85000, 70000, 60000, 50000,
                         40000, 30000, 25000, 20000, 15000, 10000,
                         7000, 5000, 3000, 2000, 1000, 500, 100])
        plev_bounds = np.array([
            [103750, 96250],
            [96250, 88750],
            [88750, 77500],
            [77500, 65000],
            [65000, 55000],
            [55000, 45000],
            [45000, 35000],
            [35000, 27500],
            [27500, 22500],
            [22500, 17500],
            [17500, 12500],
            [12500, 8500],
            [8500, 6000],
            [6000, 4000],
            [4000, 2500],
            [2500, 1500],
            [1500, 750],
            [750, 300],
            [300, 0]])
        plev_axis_id = cmor.axis(table_entry='plev19',
                                 units='Pa', length=len(plev),
                                 coord_vals=plev[:], cell_bounds=plev_bounds[:],
                                 interval=None)
        axis_ids.append(plev_axis_id)

    # 1 degree resolution latitude and longitude
    lat = np.linspace(-89.5, 89.5, 180)
    lat_bounds = np.column_stack((np.linspace(-90., 89., 180.),
                                  np.linspace(-89., 90., 180.)))
    lat_axis_id = cmor.axis(table_entry='latitude',
                            units='degrees_north', length=len(lat),
                            coord_vals=lat[:], cell_bounds=lat_bounds[:],
                            interval=None)
    axis_ids.append(lat_axis_id)

    lon = np.linspace(0.5, 359.5, 360)
    lon_bounds = np.column_stack((np.linspace(0., 359., 360.),
                                  np.linspace(1., 360., 360.)))
    lon_axis_id = cmor.axis(table_entry='longitude',
                            units='degrees_north', length=len(lon),
                            coord_vals=lon[:], cell_bounds=lon_bounds[:],
                            interval=None)
    axis_ids.append(lon_axis_id)

    #
    # Define the CMOR variable.
    #
    cmor.set_table(tables[1])
    in_missing = float(1.e20)
    if threeD:
        variable_id = cmor.variable(table_entry='ts', units='K',
                                    axis_ids=axis_ids, type='f', missing_value=in_missing)
    else:
        variable_id = cmor.variable(table_entry='ta', units='K',
                                    axis_ids=axis_ids, type='f', missing_value=in_missing)

    #
    # Write the data
    #
    if threeD:
        data_vals = np.zeros(
            (len(tvals), len(lat), len(lon)), np.float32) + 290.
    else:
        data_vals = np.zeros(
            (len(tvals),
             len(plev),
                len(lat),
                len(lon)),
            np.float32) + 290.
    try:
        print('writing...')
        cmor.write(variable_id, data_vals[:], ntimes_passed=np.shape(
            data_vals)[0])  # assuming time is the first dimension
    except Exception as e:
        raise Exception("ERROR writing data!")

    try:
        path = cmor.close(variable_id, file_name=True)
    except BaseException:
        raise Exception("ERROR closing cmor file!")

    print(path)


if __name__ == "__main__":

    opts = {'cmip_table': 'CMIP6_Amon.json',
            'outpath': 'Test',
            'table_path': 'Tables'}

    save(opts, threeD=True)
    save(opts, threeD=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Please first complete the following steps:
#
#   1. Download
#      https://github.com/PCMDI/cmip6-cmor-tables.git
#      Create a soft link cmip6-cmor-tables/Tables to ./Tables in your
#      working directory
#
#   python CMIP6Validtor ../Tables/CMIP6_Amon.json ../CMIP6/yourfile.nc
#

"""
Created on Fri Feb 19 11:33:52 2016

@author: Denis Nadeau LLNL
@co-author: Guillaume Levavasseur (IPSL) Parallelization

"""

from __future__ import print_function
import argparse
import itertools
import json
import os
import re
import sys
from argparse import ArgumentTypeError
from contextlib import contextmanager
from datetime import datetime
from multiprocessing import Pool
from uuid import uuid4 as uuid

import numpy
import netCDF4

import cmip6_cv


# =========================
# BColors()
# =========================


class BColors:
    """
    Background colors for print statements

    """

    def __init__(self):
        self.HEADER = '\033[95m'
        self.OKBLUE = '\033[94m'
        self.OKGREEN = '\033[1;32m'
        self.WARNING = '\033[1;34;47m'
        self.FAIL = '\033[1;31;47m'
        self.ENDC = '\033[0m'
        self.BOLD = '\033[1m'
        self.UNDERLINE = '\033[4m'


BCOLORS = BColors()


# =========================
# DIRAction()
# =========================
class DIRECTORYAction(argparse.Action):
    """
    Check if argparse is a directory.

    """

    def __call__(self, parser, namespace, values, option_string=None):
        prospective = values
        if not os.path.isdir(prospective):
            if not os.path.isfile(prospective):
                raise argparse.ArgumentTypeError(
                    'No such file/directory: {}'.format(prospective))

        if os.access(prospective, os.R_OK):
            setattr(namespace, self.dest, prospective)
        else:
            raise argparse.ArgumentTypeError(
                'Read access denied: {}'.format(prospective))


# =========================
# INPUTAction()
# =========================
class INPUTAction(argparse.Action):
    """
    Checks if the supplied input exists.

    """

    def __call__(self, parser, namespace, values, option_string=None):
        checked_values = [self.input_checker(x) for x in values]
        setattr(namespace, self.dest, checked_values)

    @staticmethod
    def input_checker(path):
        path = os.path.abspath(os.path.normpath(path))
        if not os.path.exists(path):
            msg = 'No such input: {}'.format(path)
            raise argparse.ArgumentTypeError(msg)
        return path


# =========================
# Collector()
# =========================
class Collector(object):
    """
    Base collector class to yield regular NetCDF files.

    :param list sources: The list of sources to parse
    :returns: The data collector
    :rtype: *iter*

    """

    def __init__(self, sources):
        self.sources = sources
        self.FileFilter = FilterCollection()
        self.PathFilter = FilterCollection()
        assert isinstance(self.sources, list)

    def __iter__(self):
        for source in self.sources:
            if os.path.isdir(source):
                # If input is a directory: walk through it and yields netCDF files
                for root, _, filenames in os.walk(source, followlinks=True):
                    if self.PathFilter(root):
                        for filename in sorted(filenames):
                            ffp = os.path.join(root, filename)
                            if os.path.isfile(ffp) and self.FileFilter(filename):
                                yield ffp
            else:
                # It input is a file: yields the netCDF file itself
                root, filename = os.path.split(source)
                if self.PathFilter(root) and self.FileFilter(filename):
                    yield source

    def __len__(self):
        """
        Returns collector length.

        :returns: The number of items in the collector.
        :rtype: *int*

        """
        s = 0
        for _ in self.__iter__():
            s += 1
        return s


class FilterCollection(object):
    """
    Regex dictionary with a call method to evaluate a string against several regular expressions.
    The dictionary values are 2-tuples with the regular expression as a string and a boolean
    indicating to match (i.e., include) or non-match (i.e., exclude) the corresponding expression.

    """
    if sys.version_info < (3, 7):
        FILTER_TYPES = (str, re._pattern_type)
    else:
        FILTER_TYPES = (str, re.Pattern)

    def __init__(self):
        self.filters = dict()

    def add(self, name=None, regex='*', inclusive=True):
        """Add new filter"""
        if not name:
            name = uuid()
        assert isinstance(regex, self.FILTER_TYPES)
        assert isinstance(inclusive, bool)
        self.filters[name] = (regex, inclusive)

    def __call__(self, string):
        return all([self.match(regex, string, inclusive=inclusive) for regex, inclusive in list(self.filters.values())])

    @staticmethod
    def match(pattern, string, inclusive=True):
        # Assert inclusive and exclusive flag are mutually exclusive
        if inclusive:
            return True if re.search(pattern, string) else False
        else:
            return True if not re.search(pattern, string) else False


# =========================
# ProcessContext()
# =========================
class ProcessContext(object):
    """
    Encapsulates the processing context/information for child process.

    :param dict args: Dictionary of argument to pass to child process
    :returns: The processing context
    :rtype: *ProcessContext*

    """

    def __init__(self, args):
        assert isinstance(args, dict)
        for key, value in list(args.items()):
            setattr(self, key, value)


# =========================
# checkCMIP6()
# =========================
class checkCMIP6(object):
    """
    Validate if a file is CMIP6 compliant and ready for publication.

    Class need to read CMIP6 Table and Controled Vocabulary file.

    As well,the class will load the EXPERIMENT json file

    Input:
        args.cmip6_table:  CMIP6 table used to creat this file,
                           variable attributes and dimensions will be controled.
        args.CV:           Controled Vocabulary "json" file.

    Output:
        outfile:      Log file, default is stdout.

    """

    def __init__(self, table_path):
        # -------------------------------------------------------------------
        #  Reset CV error switch
        # -------------------------------------------------------------------
        self.errors = 0
        # -------------------------------------------------------------------
        #  Initialize table path
        # -------------------------------------------------------------------
        self.cmip6_table_path = os.path.normpath(table_path)
        # -------------------------------------------------------------------
        #  Initialize attributes dictionaries
        # -------------------------------------------------------------------
        self.dictGbl = dict()
        self.dictVar = dict()
        # -------------------------------------------------------------------
        # call setup() to clean all 'C' internal memory.
        # -------------------------------------------------------------------
        cmip6_cv.setup(inpath="../Tables", exit_control=cmip6_cv.CMOR_EXIT_ON_WARNING)
        # -------------------------------------------------------------------
        # Set Control Vocabulary file to use (default from cmor.h)
        # -------------------------------------------------------------------
        cmip6_cv.set_cur_dataset_attribute(
            cmip6_cv.GLOBAL_CV_FILENAME,
            cmip6_cv.TABLE_CONTROL_FILENAME)
        cmip6_cv.set_cur_dataset_attribute(
            cmip6_cv.FILE_PATH_TEMPLATE,
            cmip6_cv.CMOR_DEFAULT_PATH_TEMPLATE)
        cmip6_cv.set_cur_dataset_attribute(
            cmip6_cv.FILE_NAME_TEMPLATE,
            cmip6_cv.CMOR_DEFAULT_FILE_TEMPLATE)
        cmip6_cv.set_cur_dataset_attribute(
            cmip6_cv.GLOBAL_ATT_FURTHERINFOURLTMPL,
            cmip6_cv.CMOR_DEFAULT_FURTHERURL_TEMPLATE)
        cmip6_cv.set_cur_dataset_attribute(
            cmip6_cv.CMOR_AXIS_ENTRY_FILE,
            "CMIP6_coordinate.json")
        cmip6_cv.set_cur_dataset_attribute(
            cmip6_cv.CMOR_FORMULA_VAR_FILE,
            "CMIP6_formula_terms.json")

    @staticmethod
    def _get_variable_from_filename(f):
        return f.split('_')[0]

    @staticmethod
    def _get_table_from_filename(f):
        return f.split('_')[1]

    @staticmethod
    def _check_json_table(path):
        f = open(path)
        lines = f.readlines()
        f.close()
        jsonobject = json.loads(" ".join(lines))
        if not jsonobject:
            raise argparse.ArgumentTypeError(
                'Invalid JSON CMOR table: {}'.format(path))
        return jsonobject

    def set_double_value(self, attribute):
        if cmip6_cv.has_cur_dataset_attribute(attribute):
            if isinstance(self.dictGbl[attribute], numpy.ndarray) and \
                    isinstance(self.dictGbl[attribute][0], numpy.float64):
                self.dictGbl[attribute] = self.dictGbl[attribute][0]
                cmip6_cv.set_cur_dataset_attribute(attribute, self.dictGbl[attribute])

    @staticmethod
    def is_climatology(filename, **kwargs):
        return True if filename.find('-clim') != -1 else False

    @staticmethod
    def has_3_dimensions(infile, variable, **kwargs):
        return True if len(infile.variables[variable].dimensions) == 3 else False

    @staticmethod
    def has_27_pressure_levels(infile, **kwargs):
        dim = [d for d in list(infile.dimensions.keys()) if 'plev' in d]
        return True if len(dim) == 1 and infile.dimensions[dim[0]].size == 27 else False

    @staticmethod
    def has_7_pressure_levels(infile, **kwargs):
        dim = [d for d in list(infile.dimensions.keys()) if 'plev' in d]
        return True if len(dim) == 1 and infile.dimensions[dim[0]].size == 7 else False

    @staticmethod
    def has_4_pressure_levels(infile, **kwargs):
        dim = [d for d in list(infile.dimensions.keys()) if 'plev' in d]
        return True if len(dim) == 1 and infile.dimensions[dim[0]].size == 4 else False

    @staticmethod
    def has_land_in_cell_methods(infile, variable, **kwargs):
        return True if 'land' in infile.variables[variable].cell_methods else False

    @staticmethod
    def has_variable_name(filename, **kwargs):
        return True

    def ControlVocab(self, ncfile, variable=None, print_all=True):
        """
        Check CMIP6 global attributes against Control Vocabulary file.

            1. Validate required attribute if presents and some values.
            2. Validate registered institution and institution_id
            3. Validate registered source and source_id
            4. Validate experiment, experiment_id and all attributes associated with this experiment.
                   Make sure that all attributes associate with the experiment_id found in CMIP6_CV.json
                   are set to the appropriate values.
            5. Validate grid_label and grid_resolution
            6. Validate creation time in ISO format (YYYY-MM-DDTHH:MM:SS)
            7. Validate furtherinfourl from CV internal template
            8. Validate variable attributes with CMOR JSON table.
            9. Validate parent_* attribute
           10. Validate sub_experiment_* attributes.
           11. Validate that all *_index are integers.

        """
        filename = os.path.basename(ncfile)
        # -------------------------------------------------------------------
        #  Initialize arrays
        # -------------------------------------------------------------------
        # If table_path is the table directory
        # Deduce corresponding JSON from filename
        if os.path.isdir(self.cmip6_table_path):
            cmip6_table = '{}/CMIP6_{}.json'.format(
                self.cmip6_table_path, self._get_table_from_filename(filename))
        else:
            cmip6_table = self.cmip6_table_path
        table_id = os.path.basename(os.path.splitext(cmip6_table)[0]).split('_')[1]
        # Check and get JSON table
        cmor_table = self._check_json_table(cmip6_table)
        # -------------------------------------------------------------------
        # Load CMIP6 table into memory
        # -------------------------------------------------------------------
        table = cmip6_cv.load_table(cmip6_table)
        # -------------------------------------------------------------------
        #  Deduce variable
        # -------------------------------------------------------------------
        # If variable can be deduced from the filename (Default)
        # If not variable submitted on command line with --variable is considered
        variable_id = self._get_variable_from_filename(filename)
        if not variable:
            variable = variable_id
        # -------------------------------------------------------------------
        #  Distinguish similar CMOR entries with the same out_name if exist
        # -------------------------------------------------------------------
        # Apply test on variable only if a particular treatment if required
        prepare_path = os.path.dirname(os.path.realpath(__file__))
        out_names_tests = json.loads(open(os.path.join(prepare_path, 'out_names_tests.json')).read())
        # -------------------------------------------------------------------
        #  Open file in processing
        #  The file needs to be open before the calling the test.
        # -------------------------------------------------------------------
        infile = netCDF4.Dataset(ncfile, "r")
        key = '{}_{}'.format(table_id, variable_id)
        variable_cmor_entry = None
        if key in list(out_names_tests.keys()):
            for test, cmor_entry in list(out_names_tests[key].items()):
                if getattr(self, test)(**{'infile': infile,
                                          'variable': variable,
                                          'filename': filename}):
                    # If test successfull, the CMOR entry to consider is given by the test
                    variable_cmor_entry = cmor_entry
                    break
                else:
                    # If not, CMOR entry to consider is the variable from filename or from input command-line
                    variable_cmor_entry = variable
        else:
            # By default, CMOR entry to consider is the variable from filename or from input command-line
            variable_cmor_entry = variable
        # -------------------------------------------------------------------
        #  Get variable out name in netCDF record
        #  -------------------------------------------------------------------
        # Variable record name should follow CMOR table out names
        if variable_cmor_entry not in list(cmor_table['variable_entry'].keys()):
            print(BCOLORS.FAIL)
            print("=====================================================================================")
            print("The entry " + variable_cmor_entry + " could not be found in CMOR table")
            print("=====================================================================================")
            print(BCOLORS.ENDC)
            raise KeyboardInterrupt
        variable_record_name = cmor_table['variable_entry'][variable_cmor_entry]['out_name']
        # Variable id attribute should be the same as variable record name
        # in any case to be CF- and CMIP6-compliant
        variable_id = variable_record_name
        # -------------------------------------------------------------------
        # Create a dictionary of all global attributes
        # -------------------------------------------------------------------
        self.dictGbl = infile.__dict__
        for key, value in list(self.dictGbl.items()):
            cmip6_cv.set_cur_dataset_attribute(key, value)
        # Set member_id attribute depending on sub_experiment_id and variant_label
        member_id = ""
        if "sub_experiment_id" in list(self.dictGbl.keys()):
            if self.dictGbl["sub_experiment_id"] not in ['none']:
                member_id = '{}-{}'.format(self.dictGbl['sub_experiment_id'],
                                           self.dictGbl['variant_label'])
            else:
                member_id = self.dictGbl['variant_label']
        cmip6_cv.set_cur_dataset_attribute(cmip6_cv.GLOBAL_ATT_MEMBER_ID, member_id)
        # -------------------------------------------------------------------
        # Create a dictionary of attributes for the variable
        # -------------------------------------------------------------------
        try:
            self.dictVar = infile.variables[variable_record_name].__dict__
        except BaseException:
            print(BCOLORS.FAIL)
            print("=====================================================================================")
            print("The variable " + variable_record_name + " could not be found in file")
            print("=====================================================================================")
            print(BCOLORS.ENDC)
            raise KeyboardInterrupt

        # -------------------------------------------------------------------
        # Check global attributes
        # -------------------------------------------------------------------
        self.errors += cmip6_cv.check_requiredattributes(table)
        self.errors += cmip6_cv.check_institution(table)
        self.errors += cmip6_cv.check_sourceID(table)
        self.errors += cmip6_cv.check_experiment(table)
        self.errors += cmip6_cv.check_grids(table)
        self.errors += cmip6_cv.check_ISOTime()
        self.errors += cmip6_cv.check_furtherinfourl(table)
        self.errors += cmip6_cv.check_subExpID(table)
        for attr in ['branch_time_in_child', 'branch_time_in_parent']:
            if attr in list(self.dictGbl.keys()):
                self.set_double_value(attr)
                if not numpy.issubdtype(self.dictGbl[attr], numpy.float64):
                    print(BCOLORS.FAIL)
                    print("=====================================================================================")
                    print("{} is not a double: ".format(attr), type(self.dictGbl[attr]))
                    print("=====================================================================================")
                    print(BCOLORS.ENDC)
                    self.errors += 1
        for attr in ['realization_index', 'initialization_index', 'physics_index', 'forcing_index']:
            if not numpy.issubdtype(self.dictGbl[attr], numpy.integer):
                print(BCOLORS.FAIL)
                print("=====================================================================================")
                print("{} is not an integer: ".format(attr), type(self.dictGbl[attr]))
                print("=====================================================================================")
                print(BCOLORS.ENDC)
                self.errors += 1
        self.errors += cmip6_cv.check_parentExpID(table)
        for attr in ['table_id', 'variable_id']:
            try:
                if locals()[attr] != self.dictGbl[attr]:
                    print(BCOLORS.FAIL)
                    print("=====================================================================================")
                    print("{} attribute is not consistent: ".format(attr), self.dictGbl[attr])
                    print("=====================================================================================")
                    print(BCOLORS.ENDC)
                    self.errors += 1
            except KeyError:
                print(BCOLORS.FAIL)
                print("=====================================================================================")
                print("{} attribute is missing in global attributes".format(attr))
                print("=====================================================================================")
                print(BCOLORS.ENDC)
                self.errors += 1
        # -------------------------------------------------------------------
        # Get time axis properties
        # -------------------------------------------------------------------
        # Get calendar and time units
        try:
            calendar = infile.variables['time'].calendar
            timeunits = infile.variables['time'].units
        except BaseException:
            calendar = "gregorian"
            timeunits = "days since ?"
        # Get first and last time bounds

        climatology = self.is_climatology(filename)
        if climatology:
            if cmip6_table.find('Amon') != -1:
                variable = '{}Clim'.format(variable)

        clim_idx = variable.find('Clim')
        if climatology and clim_idx != -1:
            var = [variable[:clim_idx]]

        try:
            if 'bounds' in list(infile.variables['time'].__dict__.keys()):
                bndsvar = infile.variables['time'].__dict__['bounds']
            elif 'climatology' in list(infile.variables['time'].__dict__.keys()):
                bndsvar = infile.variables['time'].__dict__['climatology']
            else:
                bndsvar = 'time_bnds'
            startimebnds = infile.variables[bndsvar][0][0]
            endtimebnds = infile.variables[bndsvar][-1][1]
        except BaseException:
            startimebnds = 0
            endtimebnds = 0

        try:
            startime = infile.variables['time'][0]
            endtime = infile.variables['time'][-1]
        except BaseException:
            startime = 0
            endtime = 0

        # -------------------------------------------------------------------
        # Setup variable
        # -------------------------------------------------------------------
        varid = cmip6_cv.setup_variable(variable_cmor_entry,
                                        cmor_table['variable_entry'][variable_cmor_entry]['units'],
                                        float(cmor_table['Header']['missing_value']),
                                        int(cmor_table['Header']['int_missing_value']),
                                        startime,
                                        endtime,
                                        startimebnds,
                                        endtimebnds)
        if varid == -1:
            print(BCOLORS.FAIL)
            print("=====================================================================================")
            print("Could not find variable {} in table {} ".format(variable_cmor_entry, cmip6_table))
            print("=====================================================================================")
            print(BCOLORS.ENDC)
            raise KeyboardInterrupt
        # -------------------------------------------------------------------
        # Check filename
        # -------------------------------------------------------------------
        self.errors += cmip6_cv.check_filename(table,
                                               varid,
                                               calendar,
                                               timeunits,
                                               filename)
        # -------------------------------------------------------------------
        # Check variable attributes
        # -------------------------------------------------------------------
        cv_attrs = cmip6_cv.list_variable_attributes(varid)
        for key in cv_attrs:
            if key == "long_name":
                continue
            if key == "comment":
                continue
            if key == "cell_measures":
                if " OR " in cv_attrs[key] or "OPT" in cv_attrs[key] or "MODEL" in cv_attrs[key] or "UGRID" in cv_attrs[key]:
                    continue
            # Is this attribute in file?
            if key in list(self.dictVar.keys()):
                # Verify that attribute value is equal to file attribute
                table_value = cv_attrs[key]
                file_value = self.dictVar[key]
                # PrePARE accept units of 1 or 1.0 so adjust the table_value
                if key == "units":
                    if (table_value == "1") and (file_value == "1.0"):
                        table_value = "1.0"
                    if (table_value == "1.0") and (file_value == "1"):
                        table_value = "1"
                if isinstance(table_value, str) and isinstance(file_value, numpy.ndarray):
                    if numpy.array([int(value) for value in table_value.split()] == file_value).all():
                        file_value = True
                        table_value = True
                if isinstance(table_value, numpy.ndarray):
                    table_value = table_value[0]
                if isinstance(file_value, numpy.ndarray):
                    file_value = file_value[0]
                if isinstance(table_value, float):
                    if abs(table_value - file_value) <= 0.00001 * abs(table_value):
                        table_value = file_value
                if key == "cell_methods":
                    idx = file_value.find(" (")
                    if idx != -1:
                        file_value = file_value[:idx]
                        table_value = table_value[:idx]
                if key == "cell_measures":
                    # Check if area and volume values from the table's cell_measures are found in the file's external_variables
                    pattern = re.compile('(?:area|volume): (\w+)')
                    values = re.findall(pattern, table_value)
                    for v in values:
                        if not re.search(r"\b{}\b".format(v), self.dictGbl['external_variables']):
                            print(BCOLORS.FAIL)
                            print("=====================================================================================")
                            print("Your file contains external_variables = \"" + self.dictGbl['external_variables'] + "\", and")
                            if len(values) == 2:
                                print("CMIP6 tables requires \"" + values[0] + "\" and \"" + values[1] + "\" in external_variables.")
                            else:
                                print("CMIP6 tables requires \"" + values[0] + "\" in external_variables.")
                            print("=====================================================================================")
                            print(BCOLORS.ENDC)
                            self.errors += 1
                        continue

                if str(table_value) != str(file_value):
                    print(BCOLORS.FAIL)
                    print("=====================================================================================")
                    print("Your file contains \"" + key + "\":\"" + str(file_value) + "\" and")
                    print("CMIP6 tables requires \"" + key + "\":\"" + str(table_value) + "\".")
                    print("=====================================================================================")
                    print(BCOLORS.ENDC)
                    self.errors += 1
            else:
                # That attribute is not in the file
                table_value = cv_attrs[key]
                if isinstance(table_value, numpy.ndarray):
                    table_value = table_value[0]
                if isinstance(table_value, float):
                    table_value = "{0:.2g}".format(table_value)
                print(BCOLORS.FAIL)
                print("=====================================================================================")
                print("CMIP6 variable " + variable + " requires \"" + key + "\":\"" + str(table_value) + "\".")
                print("=====================================================================================")
                print(BCOLORS.ENDC)
                self.errors += 1
        # Check if cell_measures is defined in the file but not in the table
        if "cell_measures" in list(self.dictVar.keys()) and "cell_measures" not in cv_attrs:
                print(BCOLORS.FAIL)
                print("=====================================================================================")
                print("Your file contains \"cell_measures\":\"" + str(self.dictVar["cell_measures"]) + "\" but")
                print("CMIP6 tables do not define \"cell_measures\".")
                print("=====================================================================================")
                print(BCOLORS.ENDC)
                self.errors += 1
        # Print final message
        if self.errors != 0:
            print(BCOLORS.FAIL + "└──> :: CV FAIL    :: {}".format(ncfile) + BCOLORS.ENDC)
            raise KeyboardInterrupt
        elif print_all:
            print(BCOLORS.OKGREEN + "     :: CV SUCCESS :: {}".format(ncfile) + BCOLORS.ENDC)


def process(source):
    # Redirect all print statements to a logfile dedicated to the current
    # process
    logfile = '/tmp/PrePARE-{}.log'.format(os.getpid())
    with RedirectedOutput(logfile):
        rc = sequential_process(source)
    # Close and return logfile
    return logfile, rc


def sequential_process(source):
    # Get context from global process env
    assert 'pctx' in list(globals().keys())
    pctx = globals()['pctx']
    try:
        # Process file
        checker = checkCMIP6(pctx.table_path)
        if pctx.variable:
            checker.ControlVocab(source, variable=pctx.variable, print_all=pctx.all)
        else:
            checker.ControlVocab(source, print_all=pctx.all)
        return 0
    except KeyboardInterrupt:
        return 1
    except Exception as e:
        print(e)
        msg = BCOLORS.WARNING
        msg += "└──> :: SKIPPED    :: {}".format(source)
        msg += BCOLORS.ENDC
        print(msg)
        return 1
    finally:
        # Close opened file
        if hasattr(checker, "infile"):
            checker.infile.close()


def initializer(keys, values):
    """
    Initialize process context by setting particular variables as global variables.

    :param list keys: Argument name list
    :param list values: Argument value list

    """
    assert len(keys) == len(values)
    global pctx
    pctx = ProcessContext({key: values[i] for i, key in enumerate(keys)})


def regex_validator(string):
    """
    Validates a Python regular expression syntax.

    :param str string: The string to check
    :returns: The Python regex
    :rtype: *re.compile*
    :raises Error: If invalid regular expression

    """
    try:
        return re.compile(string)
    except re.error:
        msg = 'Bad regex syntax: {}'.format(string)
        raise ArgumentTypeError(msg)


def processes_validator(value):
    """
    Validates the max processes number.

    :param str value: The max processes number submitted
    :return:
    """
    pnum = int(value)
    if pnum < 1 and pnum != -1:
        msg = 'Invalid processes number. Should be a positive integer or "-1".'
        raise ArgumentTypeError(msg)
    if pnum == -1:
        # Max processes = None corresponds to cpu.count() in Pool creation
        return None
    else:
        return pnum


#  =========================
#   main()
#  =========================
def main():
    parser = argparse.ArgumentParser(
        prog='PrePARE',
        description='Validate CMIP6 file for ESGF publication.')

    parser.add_argument(
        '-l', '--log',
        metavar='CWD',
        type=str,
        const='{}/logs'.format(os.getcwd()),
        nargs='?',
        help='Logfile directory. Default is the working directory.\n'
             'If not, standard output is used. Only available in multiprocessing mode.')

    parser.add_argument(
        '--variable',
        help='Specify geophysical variable name.\n'
             'If not variable is deduced from filename.')

    parser.add_argument(
        '--table-path',
        action=DIRECTORYAction,
        default=os.environ['CMIP6_CMOR_TABLES'] if 'CMIP6_CMOR_TABLES' in list(os.environ.keys()) else './Tables',
        help='Specify the CMIP6 CMOR tables path (JSON file).\n'
             'If not submitted read the CMIP6_CMOR_TABLES environment variable if exists.\n'
             'If a directory is submitted table is deduced from filename (default is "./Tables").')

    parser.add_argument(
        '--max-processes',
        metavar='4',
        type=processes_validator,
        default=4,
        help='Number of maximal processes to simultaneously treat several files.\n'
             'Set to one seems sequential processing (default). Set to "-1" seems\n'
             'all available resources as returned by "multiprocessing.cpu_count()".')

    parser.add_argument(
        '--all',
        action='store_true',
        default=False,
        help='Show all results. Default only shows error(s) (i.e., file(s) not compliant)')

    parser.add_argument(
        '--ignore-dir',
        metavar="PYTHON_REGEX",
        type=str,
        default='^.*/\.[\w]*$',
        help='Filter directories NON-matching the regular expression.\n'
             'Default ignores paths with folder name(s) starting with "."')

    parser.add_argument(
        '--include-file',
        metavar='PYTHON_REGEX',
        type=regex_validator,
        action='append',
        help='Filter files matching the regular expression.\n'
             'Duplicate the flag to set several filters.\n'
             'Default only include NetCDF files.')

    parser.add_argument(
        '--exclude-file',
        metavar='PYTHON_REGEX',
        type=regex_validator,
        action='append',
        help='Filter files NON-matching the regular expression.\n'
             'Duplicate the flag to set several filters.\n'
             'Default only exclude hidden files (with names not\n'
             'starting with ".").')

    parser.add_argument(
        'input',
        nargs='+',
        action=INPUTAction,
        help='Input CMIP6 netCDF data to validate (ex: clisccp_cfMon_DcppC22_NICAM_gn_200001-200001.nc).\n'
             'If a directory is submitted all netCDF recursively found will be validate independently.')

    # Check command-line error
    try:
        args = parser.parse_args()
    except argparse.ArgumentTypeError as errmsg:
        print(str(errmsg), file=sys.stderr)
        return 1
    except SystemExit:
        return 1
    # Get log
    logname = 'PrePARE-{}.log'.format(datetime.now().strftime("%Y%m%d-%H%M%S"))
    log = None
    if args.log:
        if not os.path.isdir(args.log):
            os.makedirs(args.log)
        log = os.path.join(args.log, logname)
    # Collects netCDF files for process
    sources = Collector(args.input)
    # Set scan filters
    file_filters = list()
    if args.include_file:
        file_filters.extend([(f, True) for f in args.include_file])
    else:
        # Default includes netCDF only
        file_filters.append(('^.*\.nc$', True))
    if args.exclude_file:
        # Default exclude hidden files
        file_filters.extend([(f, False) for f in args.exclude_file])
    else:
        file_filters.append(('^\..*$', False))
    # Init collector file filter
    for regex, inclusive in file_filters:
        sources.FileFilter.add(regex=regex, inclusive=inclusive)
    # Init collector dir filter
    sources.PathFilter.add(regex=args.ignore_dir, inclusive=False)
    nb_sources = len(sources)
    errors = 0
    # Init process context
    cctx = dict()
    cctx['table_path'] = args.table_path
    cctx['variable'] = args.variable
    cctx['all'] = args.all
    # Separate sequential process and multiprocessing
    if args.max_processes != 1:
        # Create pool of processes
        pool = Pool(processes=args.max_processes, initializer=initializer, initargs=(list(cctx.keys()), list(cctx.values())))
        # Run processes
        logfiles = list()
        progress = 0
        for logfile, rc in pool.imap(process, sources):
            progress += 1
            percentage = int(progress * 100 / nb_sources)
            msg = BCOLORS.OKGREEN + '\rCheck netCDF file(s): ' + BCOLORS.ENDC
            msg += '{}% | {}/{} files'.format(percentage, progress, nb_sources)
            sys.stdout.write(msg)
            sys.stdout.flush()
            logfiles.append(logfile)
            errors += rc
        sys.stdout.write('\r\033[K')
        sys.stdout.flush()
        # Print results from logfiles and remove them
        for logfile in set(logfiles):
            if not os.stat(logfile).st_size == 0:
                with open(logfile, 'r') as f:
                    if log:
                        with open(log, 'a+') as r:
                            r.write(f.read())
                    else:
                        sys.stdout.write(f.read())
                        sys.stdout.flush()
            os.remove(logfile)
        # Close pool of processes
        pool.close()
        pool.join()
    else:
        print('Checking data, please wait...')
        initializer(list(cctx.keys()), list(cctx.values()))
        for source in sources:
            errors += sequential_process(source)
    # Print results summary
    msg = BCOLORS.HEADER + '\nNumber of files scanned: {}'.format(nb_sources) + BCOLORS.ENDC
    if errors:
        msg += BCOLORS.FAIL
    else:
        msg += BCOLORS.OKGREEN
    msg += '\nNumber of file with error(s): {}'.format(errors) + BCOLORS.ENDC
    if log:
        with open(log, 'a+') as r:
            r.write(msg)
    print(msg)
    # Evaluate errors and exit with appropriate return code
    if errors != 0:
        if errors == nb_sources:
            # All files has error(s). Error code = -1
            sys.exit(-1)
        else:
            # Some files (at least one) has error(s). Error code = nb files with error(s)
            sys.exit(errors)
    else:
        # No errors. Error code = 0
        sys.exit(0)


@contextmanager
def RedirectedOutput(to=os.devnull):
    fd_out = sys.stdout.fileno()
    old_stdout = os.fdopen(os.dup(fd_out), 'w')
    fd_err = sys.stderr.fileno()
    old_stderr = os.fdopen(os.dup(fd_err), 'w')
    stream = open(to, 'a+')
    sys.stdout.close()
    sys.stderr.close()
    os.dup2(stream.fileno(), fd_out)
    os.dup2(stream.fileno(), fd_err)
    sys.stdout = os.fdopen(fd_out, 'w')
    sys.stderr = os.fdopen(fd_err, 'w')
    try:
        yield
    finally:
        sys.stdout.close()
        sys.stderr.close()
        os.dup2(old_stdout.fileno(), fd_out)
        os.dup2(old_stderr.fileno(), fd_err)
        sys.stdout = os.fdopen(fd_out, 'w')
        sys.stderr = os.fdopen(fd_err, 'w')


if __name__ == '__main__':
    main()



# pylint: disable = missing-docstring, invalid-name
"""
Tests for ``cmor.has_cur_dataset_attribute``.
"""
import os
import unittest

import cmor


class TestHasCurDatasetAttribute(unittest.TestCase):
    """
    Tests for ``cmor.has_cur_dataset_attribute``.
    """

    def setUp(self):
        self.logfile = 'has_cur_dataset_attribute.log'
        cmor.setup(logfile=self.logfile)
        cmor.set_cur_dataset_attribute('valid_attribute', 'valid_value')

    def test_has_cur_dataset_attribute_with_valid_attribute(self):
        self.assertTrue(cmor.has_cur_dataset_attribute('valid_attribute'))

    def test_has_cur_dataset_attribute_with_invalid_attribute(self):
        self.assertFalse(cmor.has_cur_dataset_attribute('invalid_attribute'))

    def tearDown(self):
        cmor.close()
        if os.path.isfile(self.logfile):
            os.remove(self.logfile)


if __name__ == '__main__':
    unittest.main()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.
from __future__ import print_function
import cmor
import numpy
import unittest
import os
import sys
import tempfile
import pkgutil
import netCDF4
import base_CMIP6_CV

# ==============================
#  main thread
# ==============================
def run():
    unittest.main()


class TestCase(base_CMIP6_CV.BaseCVsTest):
    def testCMIP6(self):

        nlat = 10
        dlat = 180. / nlat
        nlon = 20
        dlon = 360. / nlon
        nlev = 5
        ntimes = 5

        lats = numpy.arange(90 - dlat / 2., -90, -dlat)
        blats = numpy.arange(90, -90 - dlat, -dlat)
        lons = numpy.arange(0 + dlon / 2., 360., dlon)
        blons = numpy.arange(0, 360. + dlon, dlon)

        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        # --------------------------------------------
        # load Omon table and create masscello variable
        # --------------------------------------------
        cmor.load_table("CMIP6_Omon.json")
        itime = cmor.axis(table_entry="time", units='months since 2010',
                          coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                          cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
        ilat = cmor.axis(
            table_entry='latitude',
            coord_vals=lats,
            cell_bounds=blats,
            units='degrees_north')
        ilon = cmor.axis(
            table_entry='longitude',
            coord_vals=lons,
            cell_bounds=blons,
            units='degrees_east')
        ilev = cmor.axis(table_entry='depth_coord', length=5,
                         cell_bounds=numpy.arange(0, 12000, 2000), coord_vals=numpy.arange(0, 10000, 2000), units="m")

        ivar = cmor.variable(
            table_entry="masscello", axis_ids=[
                itime, ilev, ilat, ilon, ], units='kg/m2')

        data = numpy.random.random((ntimes, nlev, nlat, nlon)) * 100.

        cmor.write(ivar, data)
        self.delete_files += [cmor.close(ivar, True)]
        cmor.close()

        f = netCDF4.Dataset(cmor.get_final_filename(), "r")
        self.assertTrue("external_variables" in f.__dict__)
        a = f.__dict__["external_variables"]
        self.assertEqual("areacello volcello", a)
        f.close()

    @unittest.skipIf(pkgutil.find_loader("cdms2") is None, "cdms2 not installed")
    def testCMIP6_CDMS2(self):
        import cdms2

        nlat = 10
        dlat = 180. / nlat
        nlon = 20
        dlon = 360. / nlon
        nlev = 5
        ntimes = 5

        lats = numpy.arange(90 - dlat / 2., -90, -dlat)
        blats = numpy.arange(90, -90 - dlat, -dlat)
        lons = numpy.arange(0 + dlon / 2., 360., dlon)
        blons = numpy.arange(0, 360. + dlon, dlon)

        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        # --------------------------------------------
        # load Omon table and create masscello variable
        # --------------------------------------------
        cmor.load_table("CMIP6_Omon.json")
        itime = cmor.axis(table_entry="time", units='months since 2010',
                          coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                          cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
        ilat = cmor.axis(
            table_entry='latitude',
            coord_vals=lats,
            cell_bounds=blats,
            units='degrees_north')
        ilon = cmor.axis(
            table_entry='longitude',
            coord_vals=lons,
            cell_bounds=blons,
            units='degrees_east')
        ilev = cmor.axis(table_entry='depth_coord', length=5,
                         cell_bounds=numpy.arange(0, 12000, 2000), coord_vals=numpy.arange(0, 10000, 2000), units="m")

        ivar = cmor.variable(
            table_entry="masscello", axis_ids=[
                itime, ilev, ilat, ilon, ], units='kg/m2')

        data = numpy.random.random((ntimes, nlev, nlat, nlon)) * 100.

        cmor.write(ivar, data)
        self.delete_files += [cmor.close(ivar, True)]
        cmor.close()

        f = cdms2.open(cmor.get_final_filename(), "r")
        a = f.getglobal("external_variables")
        self.assertEqual("areacello volcello", a)
        f.close()

    def testCMIP6_ExternaVariablesError(self):

        print("CMIP6 External Error")
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        error_flag = cmor.dataset_json('Test/CMOR_input_example.json')
        table_id = cmor.load_table('CMIP6_6hrLev.json')
        time = cmor.axis(table_entry='time1', units='days since 2000-01-01',
                        coord_vals=numpy.array(range(1)),
                        cell_bounds=numpy.array(range(2)))
        latitude = cmor.axis(table_entry='latitude', units='degrees_north',
                            coord_vals=numpy.array(range(5)),
                            cell_bounds=numpy.array(range(6)))
        longitude = cmor.axis(table_entry='longitude', units='degrees_east',
                            coord_vals=numpy.array(range(5)),
                            cell_bounds=numpy.array(range(6)))
        plev3 = cmor.axis(table_entry='plev3', units='Pa',
                        coord_vals=numpy.array([85000., 50000., 25000.]))
        axis_ids = [longitude, latitude, plev3, time]
        ua_var_id = cmor.variable(table_entry='ua', axis_ids=axis_ids,
                                units='m s-1')
        ta_var_id = cmor.variable(table_entry='ta', axis_ids=axis_ids,
                                units='K')
        data = numpy.random.random(75)
        reshaped_data = data.reshape((5, 5, 3, 1))

        # This doesn't:
        cmor.write(ta_var_id, reshaped_data)
        cmor.write(ua_var_id, reshaped_data)
        fname_ta = cmor.close(ta_var_id, file_name=True)
        fname_ua = cmor.close(ua_var_id, file_name=True)
        self.delete_files += [cmor.close(ta_var_id, True)]
        self.delete_files += [cmor.close(ua_var_id, True)]
        cmor.close()

        f = netCDF4.Dataset(fname_ta, "r")
        self.assertTrue("external_variables" in f.__dict__)
        a = f.__dict__["external_variables"]
        self.assertEqual("areacella", a)
        f.close()
        f = netCDF4.Dataset(fname_ua, "r")
        self.assertTrue("external_variables" not in f.__dict__)
        f.close()

    @unittest.skipIf(pkgutil.find_loader("cdms2") is None, "cdms2 not installed")
    def testCMIP6_ExternaVariablesError_CDMS2(self):
        import cdms2

        print("CMIP6 External Error")
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        error_flag = cmor.dataset_json('Test/CMOR_input_example.json')
        table_id = cmor.load_table('CMIP6_6hrLev.json')
        time = cmor.axis(table_entry='time1', units='days since 2000-01-01',
                        coord_vals=numpy.array(range(1)),
                        cell_bounds=numpy.array(range(2)))
        latitude = cmor.axis(table_entry='latitude', units='degrees_north',
                            coord_vals=numpy.array(range(5)),
                            cell_bounds=numpy.array(range(6)))
        longitude = cmor.axis(table_entry='longitude', units='degrees_east',
                            coord_vals=numpy.array(range(5)),
                            cell_bounds=numpy.array(range(6)))
        plev3 = cmor.axis(table_entry='plev3', units='Pa',
                        coord_vals=numpy.array([85000., 50000., 25000.]))
        axis_ids = [longitude, latitude, plev3, time]
        ua_var_id = cmor.variable(table_entry='ua', axis_ids=axis_ids,
                                units='m s-1')
        ta_var_id = cmor.variable(table_entry='ta', axis_ids=axis_ids,
                                units='K')
        data = numpy.random.random(75)
        reshaped_data = data.reshape((5, 5, 3, 1))

        # This doesn't:
        cmor.write(ta_var_id, reshaped_data)
        cmor.write(ua_var_id, reshaped_data)
        fname_ta = cmor.close(ta_var_id, file_name=True)
        fname_ua = cmor.close(ua_var_id, file_name=True)
        self.delete_files += [cmor.close(ta_var_id, True)]
        self.delete_files += [cmor.close(ua_var_id, True)]
        cmor.close()

        f = cdms2.open(fname_ta, "r")
        a = f.getglobal("external_variables")
        self.assertEqual("areacella", a)
        f.close()
        f = cdms2.open(fname_ua, "r")
        a = f.getglobal("external_variables")
        self.assertEqual(None, a)
        f.close()


if __name__ == '__main__':
    run()
